--[[
The BlacklistPolicy has a cache of bad chat messages. The policy lookup that cache on every incoming message.
If the message is on the black list, then the message will be dropped and the timeout for the corresponding cache entry
will be reseted to the default (for example 5 minutes).

This policy does not evaluates the message text itself. It's a bouncer. Any message who was blocked by any policy 
is added to the black list automatically on the train path.
]]

BlacklistPolicy = class();

-- Initializes a new BlacklistPolicy instance
function BlacklistPolicy:Constructor(filterData)
	self.Name = "BlacklistPolicy";
	self.Data = filterData;
end

-- Executes this policy
function BlacklistPolicy:Execute(message, channel, username, text, bag)
	self:CleanupBlacklist();

	if self.Data.BlackList[message] ~= nil then
		-- Message is on the black list
		return true;
	end
end

-- Train this policy
function BlacklistPolicy:Apply(message, channel, username, text, blocked, policyName, bag)
	if blocked == true then
		if self.Data.BlackList[message] then
			-- update
			self.Data.BlackList[message].Timestamp = Turbine.Engine:GetGameTime();

			Events.FireEvent(self, self.Updated, {});
		else
			-- add to black list
			self.Data.BlackList[message] = {
				["Timestamp"] = Turbine.Engine:GetGameTime(),
				["Origin"] = policyName,
			};

			Events.FireEvent(self, self.Changed, {});
		end
	end
end

-- Cleanup the black list, remove outdated entries
function BlacklistPolicy:CleanupBlacklist()
	local count = 0;
	local modified = false;

	for key, entry in pairs(self.Data.BlackList) do
		if Turbine.Engine:GetGameTime() - entry.Timestamp > self.Data.BlackListTimeMax then
			self.Data.BlackList[key] = nil;
			modified = true;
		else
			count = count + 1;
		end
	end

	-- Seems to be very full, let's clear the black list
	if count > self.Data.BlackListThreshold then
		self.Data.BlackList = {};
		modified = true;
	end

	if modified then
		Events.FireEvent(self, self.Changed, {});
	end
end

-- Removes all entries from the blacklist
function BlacklistPolicy:Clear()
	local modified = false;

	for key, _ in pairs(self.Data.BlackList) do
		self.Data.BlackList[key] = nil;
		modified = true;
	end
	
	if modified then
		Events.FireEvent(self, self.Changed, {});
	end
end

-- Fires if an existing entry was updated
function BlacklistPolicy:Updated(args)
end

-- Fires if the blacklist was modified (an entry was added or removed)
function BlacklistPolicy:Changed(args)
end
